from pymediainfo import MediaInfo
from typing import cast, Any, Dict, Sequence


MediaTrack = Dict[str, Any]
MediaInfoList = Sequence[MediaTrack]


def get_media_info_list(filename: str) -> MediaInfoList:
    media_info = MediaInfo.parse(filename)
    return cast(MediaInfoList, [x.to_data() for x in media_info.tracks])



import json
import os

from pathlib import Path
from typing import cast, Any, Dict, Sequence

from .attribute import Attribute, SingleAttribute, CompositeAttribute
from .preference import Preference, SinglePreference
from .level import Level, DEFAULT_LEVEL

Config = Dict[str, Any]
ConfigAttribute = Dict[str, Any]
ConfigPreference = Dict[str, Any]
LevelMap = Dict[str, Level]


def get_preference(config_preference: ConfigPreference, level_map: LevelMap, render: str) -> Preference:
    return SinglePreference(
        title=render % (config_preference["title"]),
        pattern=config_preference.get("pattern", config_preference["title"]),
        level=level_map[config_preference["level"]],
    )


def get_single_attribute(config_attribute: ConfigAttribute, level_map: LevelMap) -> Attribute:
    default_level = (
        level_map[config_attribute["default_level"]] if "default_level" in config_attribute else DEFAULT_LEVEL
    )
    attribute = SingleAttribute(
        title=config_attribute["title"],
        render=config_attribute.get("render", "%s"),
        preferences=[],
        default_level=default_level,
        track_type=config_attribute["track_type"],
        track_attribute=config_attribute["track_attribute"],
    )
    attribute.preferences = [
        get_preference(x, level_map, attribute.render) for x in config_attribute.get("preferences", [])
    ]
    return attribute


def get_composite_attribute(config_attribute: ConfigAttribute, level_map: LevelMap) -> Attribute:
    default_level = (
        level_map[config_attribute["default_level"]] if "default_level" in config_attribute else DEFAULT_LEVEL
    )
    attribute = CompositeAttribute(
        title=config_attribute["title"],
        render=config_attribute.get("render", "%s"),
        attributes=[],
        preferences=[],
        default_level=default_level,
    )
    attribute.attributes = [get_attribute(x, level_map) for x in config_attribute.get("attributes", [])]
    attribute.preferences = [
        get_preference(x, level_map, attribute.render) for x in config_attribute.get("preferences", [])
    ]
    return attribute


def get_attribute(config_attribute: ConfigAttribute, level_map: LevelMap) -> Attribute:
    return (
        get_composite_attribute(config_attribute, level_map)
        if "attributes" in config_attribute
        else get_single_attribute(config_attribute, level_map)
    )


def get_default_config() -> Config:
    path = Path(__file__)
    config = path.parent.joinpath("default_config.json")
    return cast(Config, json.loads(config.read_text()))


def save_config(config: Config, path: Path) -> None:
    dir = path.parent
    if not dir.is_dir():
        os.makedirs(str(dir))
    config = get_default_config()
    path.write_text(json.dumps(config, indent=4))


def load_config(path: Path) -> Config:
    return cast(Config, json.loads(path.read_text()))


def make_attributes(config: Config = get_default_config()) -> Sequence[Attribute]:
    level_map = {}
    for name, options in config["levels"].items():
        level_map[name] = Level(color=options["color"], flag=options["flag"])

    attributes = []

    for config_attribute in config["attributes"]:
        attribute = get_attribute(config_attribute, level_map)
        attributes.append(attribute)

    return attributes


def get_attributes(path: Path) -> Sequence[Attribute]:
    if path.exists():
        config = load_config(path)
    else:
        config = get_default_config()
        save_config(config, path)

    return make_attributes(config)



import re

from abc import abstractmethod, ABC
from dataclasses import dataclass
from .level import Level, DEFAULT_LEVEL


class Preference(ABC):
    @abstractmethod
    def get_title(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def match(self, target: str) -> bool:
        raise NotImplementedError

    @abstractmethod
    def __hash__(self) -> int:
        raise NotImplementedError

    @abstractmethod
    def is_flagged(self) -> bool:
        raise NotImplementedError

    @abstractmethod
    def render(self, input: str) -> str:
        raise NotImplementedError


@dataclass
class SinglePreference(Preference):
    title: str
    pattern: str
    level: Level = DEFAULT_LEVEL

    def get_title(self) -> str:
        return self.title

    def match(self, target: str) -> bool:
        if self.pattern.isalnum():
            return target == self.pattern
        return bool(re.match(self.pattern, target))

    def __hash__(self) -> int:
        return hash(self.title)

    def is_flagged(self) -> bool:
        return self.level.flag

    def render(self, input: str) -> str:
        return self.level.render(input)



import click
import json
import os

from appdirs import user_config_dir, user_cache_dir
from pathlib import Path
from typing import Dict, List, Sequence

from .attribute import Attribute
from .config import get_attributes
from .db import get_connection
from .preference import Preference
from .progress import show_progress
from .quality import Quality
from .video import SingleVideo, Video

APP_NAME = "videoprof"
TAB = "\t"


def render_badge(preference: Preference) -> str:
    prefix = "\u2612 " if preference.is_flagged() else "\u2713 "
    return preference.render(prefix + preference.get_title())


def render_cluster(qualities: Sequence[Quality]) -> str:
    cluster = [
        render_badge(quality.preference)
        for quality in sorted(qualities, key=lambda x: x.attribute.get_title())
    ]
    return f"{' '.join(cluster)}"


def show_summary(attributes: Sequence[Attribute], videos: Sequence[Video]) -> None:
    preference_counts: Dict[Preference, int] = {}

    def preference_summary(preference: Preference) -> str:
        count = preference_counts.get(preference, 0)
        return f"{preference.get_title()}:{TAB}{preference.render(str(count))}"

    def attribute_summary(attribute: Attribute) -> str:
        counts = "\t".join([preference_summary(preference) for preference in attribute.get_preferences()])
        return f"{attribute.get_title()}{TAB}{counts}"

    for video in videos:
        for quality in video.get_qualities():
            preference_counts[quality.preference] = preference_counts.get(quality.preference, 0) + 1

    for attribute in attributes:
        print(attribute_summary(attribute))


def show_flags(videos: Sequence[Video]) -> None:
    for video in sorted(videos, key=lambda item: item.get_path()):
        if video.is_flagged():
            print(f"{video.get_path()}:{TAB}{render_cluster(video.get_qualities())}")


def show_directory_summary(directories: Dict[Path, List[Video]], only_flagged: bool) -> None:
    for directory, videos in sorted(directories.items(), key=lambda item: item[0]):
        flag_count = 0
        video_count = 0
        qualities: List[Quality] = []

        for video in videos:
            video_count += 1
            if video.is_flagged():
                flag_count += 1

            for quality in video.get_qualities():
                if quality not in qualities:
                    qualities.append(quality)

        if not only_flagged or flag_count:
            print(f"{directory}:{TAB}{flag_count}/{video_count}{TAB}{render_cluster(qualities)}")


@click.command()
@click.argument("sources", nargs=-1)
@click.option(
    "--config",
    default=os.path.join(user_config_dir(), APP_NAME, "config.json"),
    help="JSON configuration file",
)
@click.option(
    "--cache",
    default=os.path.join(user_cache_dir(), APP_NAME, "cache.db"),
    help="SQLite cache file",
)
@click.option("-s", "--summary", is_flag=True, default=True, help="Turn off attribute summary")
@click.option("-f", "--flags", is_flag=True, default=False, help="Show individual video flags")
@click.option("-d", "--dir-summaries", is_flag=True, default=False, help="Show directory summary")
@click.option(
    "-o",
    "--only-flagged",
    is_flag=True,
    default=False,
    help="Show directory summary for only flagged directories",
)
@click.option(
    "-m",
    "--media-info",
    is_flag=True,
    default=False,
    help="Show media info for the first found video and exit",
)
@click.option("-p", "--dir-depth", default=1, help="Directory depth for summaries")
def main(
    sources: Sequence[str],
    config: str,
    cache: str,
    summary: bool,
    flags: bool,
    dir_summaries: bool,
    only_flagged: bool,
    media_info: bool,
    dir_depth: int,
) -> None:
    attributes = get_attributes(Path(config))
    connection = get_connection(Path(cache))
    videos: List[Video] = []
    directories: Dict[Path, List[Video]] = {}

    for src in sources:
        origin = Path(src)
        dir_paths = origin.glob("/".join(["*" for x in range(dir_depth)]))

        for dir in dir_paths:
            directories[dir] = []
            src_paths = dir.glob("**/*")
            for path in src_paths:
                if path.is_file():
                    video = SingleVideo(path=path)
                    if media_info:
                        print(json.dumps(video.get_cached_media_info_list(connection), indent=4))
                        exit(0)
                    videos.append(video)
                    directories[dir].append(video)

    show_progress(videos, lambda video: video.analyze(attributes, connection))

    if flags:
        show_flags(videos)
        print("")

    if dir_summaries:
        show_directory_summary(directories, only_flagged)
        print("")

    if summary:
        show_summary(attributes, videos)
        print("")



import math
import sys
from typing import Callable, Sequence, TypeVar

T = TypeVar("T")


def show_progress(items: Sequence[T], process: Callable[[T], None]) -> None:
    for x in range(0, len(items)):
        progress = math.trunc(10000 * (x + 1) / len(items)) / 100
        print(f"> {progress}%  ", end="\r", flush=True, file=sys.stderr)
        process(items[x])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['videoprof']

package_data = \
{'': ['*']}

install_requires = \
['appdirs>=1.4.4,<2.0.0',
 'click>=7.1.2,<8.0.0',
 'colored>=1.4.2,<2.0.0',
 'pymediainfo>=5.0.3,<6.0.0']

entry_points = \
{'console_scripts': ['videoprof = videoprof.videoprof:main']}

setup_kwargs = {
    'name': 'videoprof',
    'version': '0.1.0',
    'description': 'Profile various attributes of local videos like resolution, codec, container, audio channels, and more!',
    'long_description': None,
    'author': 'Christian Lent',
    'author_email': 'christian@lent.us',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
